const polyName = "Polygon Main Net";
const polyChainID = 137;
const polyScanBase = 'https://polygonscan.com/tx/';

const polyTestName = "Polygon Test Net";
const polyTestChainID = 80001;
const polyScanTestBase = 'https://mumbai.polygonscan.com/tx/';

const targetChain = {
    name: polyTestName,
    chainId: polyTestChainID,
    explorerBase : polyScanTestBase
}

const tokens = {
    blast_sepolia : {
        '0x6E378CE4D043a0F28601be22b52F787502aD5D28' : {
            native : false
        },
        '0x4649CDB89826c58fe5394E80e085C239BBbC1a76' : {
            native : true
        }      
    }
}

const onDisconnected = new Event("onDisconnected");

const swapTemplate = `<div class="flex column crosscenter medpad largegap">          
<div class="swapbox flex column medgap largepad">
    <div style="justify-content:space-between;" class="fullsize flex crosscenter">
        <span class="highlighttext">Sell</span>
        <span id="balanceA" class="balancebox">Balance:??</span>
    </div>
    <div class="fullsize flex crosscenter medgap">
        <span id="aToken">??</span>
        <input id="tokenSwapA" type="number" class="flex-fill space-mono-regular orange-border">
    </div>
    <div style="justify-content:space-between;" class="fullsize flex crosscenter medgap">
        <button class="glowborder centertext fitcontent medpad" data-type="a" data-per="25">25%</button>
        <button class="glowborder centertext fitcontent medpad" data-type="a" data-per="50">50%</button>
        <button class="glowborder centertext fitcontent medpad" data-type="a" data-per="75">75%</button>
        <button class="glowborder centertext fitcontent medpad" data-type="a" data-per="100">100%</button>
    </div>
</div>
<div id="swapDirection" class="shadow">
    <div class="chevron"></div>
</div>
<div class="swapbox flex column medgap largepad">
    <div style="justify-content:space-between;" class="fullsize flex crosscenter">
        <span class="highlighttext">Buy</span>
        <span id="balanceB" class="balancebox">Balance:??</span>
    </div>
    <div class="fullsize flex crosscenter medgap">
        <span id="bToken">??</span>
        <input id="tokenSwapB" type="number" class="flex-fill space-mono-regular orange-border">
    </div>
</div>
<div class="swapbox flex column medgap largepad">            
    <div style="justify-content:space-between;" class="fullsize flex crosscenter medgap">
        <span>Max Slippage</span>
        <input id="slippage" style="width: 50px; font-size: 1em;" placeholder="%">
    </div>
    <div style="justify-content:space-between;" class="fullsize flex crosscenter medgap">
        <span>Min Recieved</span>
        <span>0.0123</span>
    </div>
</div>   
<div id="quoteWait" style="display: none;">
    <span>Fetching quote</span>
    <img src="img/PulseArcWait.gif" style="width: 1em; height: 1em;">
</div>
<button id="swapBtn" class="glowborder subtitle centertext fitcontent medpad">
    <span style="--n:1">S</span>
    <span style="--n:2">w</span>
    <span style="--n:3">a</span>
    <span style="--n:3">p</span>
</button>
<div id=swapError style="display: none;"></div>
</div>`;

export { swapTemplate }

var provider = null;
var signer = null;

var v2routerContract = null;
var v2factoryContract = null;

//Eth Act
let currentAccount = null;
let currentChain = null;

//UI
let icon = document.getElementById('connectIcon');
let cnct = document.getElementById('connect');
let cnctTxt = document.getElementById('connectText');


function groupBy (array, key) {
    return array.reduce(function(rv, x) {
      (rv[x[key]] = rv[x[key]] || []).push(x);
      return rv;
    }, {});
  };
  
function merge(arr, mergeArr){
    return arr?.map((x, i) => { return { id: x, type: mergeArr[i] }; });
}

function find(array, key, value){
  let idx = array.findIndex(item => item[key] == value);
  return idx == -1 ? '' : array[idx];
}

export { groupBy, merge, find }
function delay(milliSeconds) {
    return new Promise(resolve => {
        setTimeout(() => { resolve('') }, milliSeconds);
    })
}

export { delay }

async function loadJson(jsonfile){
    return fetch(`./${jsonfile}`)
            .then((response) => { return response.json(); });
}

export { loadJson }
function secToTimestamp(totalSeconds) {
    totalSeconds = Number(totalSeconds);
    let days = Math.floor(totalSeconds / (3600*24));
    let hours = Math.floor(totalSeconds % (3600*24) / 3600);
    let minutes = Math.floor(totalSeconds % 3600 / 60);
    let seconds = Math.floor(totalSeconds % 60);

    return `${days}d ${hours}:${(minutes < 10 ? `0${minutes}` : minutes)}:${(seconds < 10 ? `0${seconds}` : seconds)}`;
}

export { secToTimestamp }

window.secToTimestamp = secToTimestamp;
function toggleAttrib(elementId, attribute)
{
    let element = document.getElementById(elementId);
    if (element) { element.toggleAttribute(attribute); }
}

function removeAttrib(elementId, attribute)
{
    let element = document.getElementById(elementId);
    if (element) { element.removeAttribute(attribute); }
}

function addAttrib(elementId, attribute)
{
    let element = document.getElementById(elementId);
    if (element) { element.setAttribute(attribute, ''); }
}

function setAttrib(elementId, attribute, value)
{
    let element = document.getElementById(elementId);
    if (element) { element.setAttribute(attribute, value); }
}

function getAttrib(elementId, attribute)
{
    let element = document.getElementById(elementId);
    if (element) { return element.getAttribute(attribute); }
}

export { toggleAttrib, removeAttrib, addAttrib, setAttrib, getAttrib }

window.toggleAttrib = toggleAttrib;
window.removeAttrib = removeAttrib;
window.addAttrib = addAttrib;
window.setAttrib = setAttrib;
window.getAttrib = getAttrib;
function create(elementType, attributes)
{
    let element = document.createElement(elementType);
    attributes.forEach(attribute => {
        element.setAttribute(attribute.name, attribute.value);
    });
    return element;
}

function wait(element){
    element.innerHTML = '<span class="lds-dual-ring icon block"></span>';
}

export { create, wait }

window.create = create;
window.wait = wait;
function addCss(elem, css){
    let elementsCss = document.createElement("link");
    elementsCss.setAttribute("rel", "stylesheet");
    elementsCss.setAttribute("href", css);
    elem.appendChild(elementsCss);
}

export { addCss }


class BaseElement extends HTMLElement {
    
    static get observedAttributes() {
        return ['css'];
    }

    get css() { return this.getAttribute('css'); }
    set css(value) { this.setAttribute('css', value); }

    loaded;
    get isLoaded() { return this.loaded; }
    set isLoaded(value) { this.loaded = value; }

    root;
    get shadowRoot() { return this.root; }
    set shadowRoot(value) { this.root = value; }

    constructor(){ 
        super();
        this.isLoaded = false;
    }

    loadCss(){
        this.css.split(' ').forEach(link => { addCss(this.shadowRoot, link) }); 
    }
}

export { BaseElement }
class Minimap extends HTMLElement {  

    static get observedAttributes() {
        return ['map-view', 'template-id', 'auto-load'];
    }

    root;
    get shadowRoot() { return this.root; }
    set shadowRoot(value) { this.root = value; }

    loaded;

    constructor(){ 
        super();
    }

    connectedCallback() {
        this.root = this.attachShadow({ mode: "open" });
        if(this.getAttribute('auto-load')){
            this.loadTemplate();
        }
    }

    loadTemplate(){
        if(!this.loaded){
            let templateId = this.getAttribute('template-id');
            if(templateId)
            {
                let templateContent = document.getElementById(templateId).content;
                this.root.appendChild(templateContent.cloneNode(true));
                this.loaded = true;
            }
        }
    }

    show() { this.setAttribute('map-view', ''); }
    hide() { this.removeAttribute('map-view'); }
}

customElements.define('ohl-minimap', Minimap);









class Swap extends BaseElement {  

    slippage;

    tokenA;
    tokenAName;
    tokenSwapA;
    balanceA;
    cachebalanceA;

    swapDirection;

    tokenB;
    tokenBName;
    tokenSwapB;
    balanceB;
    cachebalanceB;

    quoteWait;

    swapBtn;
    swapError;

    constructor(){ 
        super();
    }

    connectedCallback() {
        super.shadowRoot = this.attachShadow({ mode: "open" });
        this.loadTemplate();
    }

    loadTemplate(){
        if(!this.loaded){
                super.shadowRoot.innerHTML = swapTemplate;
                super.loadCss();

                this.slippage = this.root.getElementById('slippage');
                this.slippage.value = 1;

                this.tokenAName = this.root.getElementById('aToken');
                this.tokenSwapA = this.root.getElementById('tokenSwapA');
                this.tokenSwapA.addEventListener('input', this.loadQuote.bind(this));
                this.balanceA = this.root.getElementById('balanceA');

                this.swapDirection = this.root.getElementById('swapDirection');
                this.swapDirection.addEventListener("click", this.changeDirection.bind(this));

                this.tokenBName = this.root.getElementById('bToken');
                this.tokenSwapB = this.root.getElementById('tokenSwapB');
                this.balanceB = this.root.getElementById('balanceB');

                let perButtons = this.root.querySelectorAll('button[data-per]');
                perButtons.forEach(btn => {
                    btn.addEventListener('click', this.inputPercent.bind(this, btn));
                });

                this.quoteWait = this.root.getElementById('quoteWait');

                this.swapBtn = this.root.getElementById('swapBtn');
                this.swapBtn.innerText = currentAccount ? "Swap" : "Connect";
                this.swapBtn.addEventListener('click', this.swap.bind(this));
                this.swapError = this.root.getElementById('swapError');
                
                window.addEventListener('onConnected', event => this.connected(event.detail));
                window.addEventListener('onDisconnected', this.disconnected.bind(this));

                super.loaded = true;
                console.log("Swap loaded");
        }
    }

    async quote(){
        if(!currentAccount){
            //this.swapBtn.disabled = "disabled";
            this.swapBtn.innerText = "Connect";
            this.quoteWait.style.display = 'none';
            return;
        }
    
        this.swapBtn.disabled = "";
        this.swapError.innerText = "";
        this.swapError.style.display = "none";
    
        let amountA = asUnit(this.tokenSwapA.value, 'ether');
    
        let quote = await v2routerContract.amountsOut(amountA, [this.tokenA.address, this.tokenB.address]);
    
        let slip = 1 - ((this.slippage.value ?? 0) / 100);
        this.tokenSwapB.value = (fromWei(quote[1]) * slip).toFixed(18);
    
        if(!tokens.blast_sepolia[this.tokenA.address].native) {
            let approved = await this.tokenA.allowance(currentAccount, v2routerContract.address);
            this.swapBtn.innerText = approved.lt(amountA) ? "Approve" : "Swap";
        } else {
            this.swapBtn.innerText = "Swap";
        }
    
        this.quoteWait.style.display = 'none';
    }
    
    async swap(){
        if(!currentAccount){
            await connect();
            return;
        }

        let amountA = asUnit(this.tokenSwapA.value, 'ether');
        let amountB = asUnit(this.tokenSwapB.value, 'ether');

        let btnText = this.swapBtn.innerText;

        try
        {
            if(tokens.blast_sepolia[this.tokenA.address].native){
                this.swapBtn.disabled = "disabled";
                this.swapBtn.innerText = "Swapping...";
    
                let tx = await v2routerContract.SwapEthForToken(
                    currentAccount, 
                    amountA, 
                    amountB, 
                    [this.tokenA.address, this.tokenB.address]
                );
                await tx.wait();
    
                this.swapBtn.disabled = "";
                this.swapBtn.innerText = "Swap";
                this.swapError.style.display = 'none';
            }
            else {
                let approved = await this.tokenA.allowance(currentAccount, v2routerContract.address);
    
                if(approved.lt(amountA)){
                    this.swapBtn.disabled = "disabled";
                    this.swapBtn.innerText = "Approving...";
    
                    let tx = await this.tokenA.approve(v2routerContract.address, amountA);
                    await tx.wait();
    
                    this.swapBtn.disabled = "";
                    this.swapBtn.innerText = "Swap";
                    this.swapError.style.display = 'none';
                } else {
                    this.swapBtn.disabled = "disabled";
                    this.swapBtn.innerText = "Swapping...";
    
                    let tx = await v2routerContract.SwapTokenForEthwFee(
                        currentAccount, 
                        amountA, 
                        amountB, 
                        [this.tokenA.address, this.tokenB.address]
                    );
                    await tx.wait();
    
                    this.swapBtn.disabled = "";
                    this.swapBtn.innerText = "Swap";
                    this.swapError.style.display = 'none';
                }    
            }
        } catch (err) {
            this.swapError.innerText = err?.data?.message ?? err?.message;
            this.swapError.style.display = 'unset';
    
            this.swapBtn.innerText = btnText;
            this.swapBtn.disabled = "";
        }
    }
    
    changeDirection(){
        console.log("changing");

        let atkn = this.tokenA;
        this.tokenA = this.tokenB;
        this.tokenB = atkn;

        let aName = this.tokenAName.innerText;
        let aBalance = this.balanceA.innerText;
        let aCache = this.cachebalanceA;

        this.tokenAName.innerText = this.tokenBName.innerText;
        this.balanceA.innerText = this.balanceB.innerText;
        this.cachebalanceA = this.cachebalanceB;

        this.tokenBName.innerText = aName;
        this.balanceB.innerText = aBalance;
        this.cachebalanceB = aCache;
    }
    
    typewatch = function(){
        var timer = 0;
        return function(callback, ms){
            clearTimeout (timer);
            timer = setTimeout(callback, ms);
        }  
    }();
    
    loadQuote(){
        this.quoteWait.style.display = 'unset';
        this.typewatch(this.quote.bind(this), 500);
    }

    async connected(account){
        console.log(`swap detected connection to ${account}`);
        initializeContracts();
        await this.loadPairs();
        this.swapBtn.innerText = "Swap";
    }

    disconnected(){
        console.log("swap detected disconnection");
        this.swapBtn.innerText = "Connect";
    }

    async inputPercent(sender){
        let percentage = sender.getAttribute('data-per');
        if(sender.getAttribute('data-type') == 'a'){
            this.tokenSwapA.value = fromWei(this.cachebalanceA) * (percentage / 100);
        }
        await this.quote();
    }

    async loadBalances(){
        if(currentAccount)
        {
            this.cachebalanceA = (tokens.blast_sepolia[this.tokenA.address].native ? await nativeBalance() : await this.tokenA.balanceOf(currentAccount)) ?? 0;
            this.balanceA.innerText = `Balance: ${fromWei(this.cachebalanceA)}`;
            this.cachebalanceB = (tokens.blast_sepolia[this.tokenB.address].native ? await nativeBalance() : await this.tokenB.balanceOf(currentAccount)) ?? 0;
            this.balanceB.innerText = `Balance: ${fromWei(this.cachebalanceB)}`;
        }
    }

    async loadPairs(){
        if(currentAccount && v2routerContract) {
            let tokenEntries = Object.entries(tokens.blast_sepolia);
            if(tokenEntries.length >= 2){
                await this.loadA(tokenEntries[0]);
                await this.loadB(tokenEntries[1]);
            }
        }
    }

    async loadA(tokenEntry){
        let [address, tokenSettings] = tokenEntry;
        this.tokenA = new ERC20(address);
        this.tokenAName.innerText = await this.tokenA.symbol();
        this.cachebalanceA = (tokenSettings.native ? await nativeBalance() : await this.tokenA.balanceOf(currentAccount)) ?? 0;
        this.balanceA.innerText = `Balance: ${fromWei(this.cachebalanceA)}`;
    }

    async loadB(tokenEntry){
        let [address, tokenSettings] = tokenEntry;
        this.tokenB = new ERC20(address);
        this.tokenBName.innerText = await this.tokenB.symbol();
        this.cachebalanceB = (tokenSettings.native ? await nativeBalance() : await this.tokenB.balanceOf(currentAccount)) ?? 0;
        this.balanceB.innerText = `Balance: ${fromWei(this.cachebalanceB)}`;
    }
}

customElements.define('ohl-swap', Swap);
class Timer extends HTMLElement {  

    static get observedAttributes() {
        return ['end-time', 'timer-interval'];
    }

    root;
    get shadowRoot() { return this.root; }
    set shadowRoot(value) { this.root = value; }

    timer;
    get timerElement() { return this.timer; }
    set timerElement(value) { this.timer = value; }

    get endTime() { return this.getAttribute('end-time'); }
    set endTime(value) { this.setAttribute('end-time', value); }

    get interval() { return this.getAttribute('timer-interval'); }
    set interval(value) { this.setAttribute('timer-interval', value); }

    constructor(){ 
        super();
        
        this.root = this.attachShadow({ mode: "open" });
        this.timer = document.createElement("span");
        this.root.appendChild(this.timer);
    
        this.syncTime();
    }

    attributeChangedCallback(name, oldValue, newValue) {
        if (name === 'end-time' || name === 'timer-interval') {
            this.syncTime();
        }
    }

    syncTime(){
        this.updateTime();
        if(this.endTime && Date.now() < this.endTime){
            clearInterval(this.timerInterval);
            this.timerInterval = setInterval(this.updateTime.bind(this), this.interval ?? 1000);
        }
    }

    updateTime(){
        console.log("updating time");
        this.timer.textContent = !this.endTime || Date.now() >= this.endTime ? '00:00:00' : new Date(this.endTime - Date.now()).toISOString().substring(8, 19).replace('T', ' days ');
    }

    start() { this.syncTime(); }
    stop(){ if(this.timerInterval) { clearInterval(this.timerInterval); } }
    
}

customElements.define('ohl-timer', Timer);
class PromiseCap {   
    maxExec = 20;
    rate = 1000;

    execCount = 0;

    constructor() { setInterval(this.resetExec.bind(this), this.rate); }
    resetExec() { this.execCount = 0; }
}

export { promiseCap };

const promiseCap = new PromiseCap();

class BaseContract {   
    provider;
    contract;
    address;
    abi;

    constructor(caddress, cabi)
    { 
        this.address = caddress;
        this.abi = cabi;
        this.loadContract(); 
    }

    loadContract(){
        if (window.ethereum) {
            this.provider = provider ?? new ethers.providers.Web3Provider(window.ethereum);
            this.contract = new ethers.Contract(this.address, this.abi, this.provider);
        }
    }
}

export { BaseContract }


let erc20abi = [
    {
        "constant": true,
        "inputs": [],
        "name": "name",
        "outputs": [
            {
                "name": "",
                "type": "string"
            }
        ],
        "payable": false,
        "stateMutability": "view",
        "type": "function"
    },
    {
        "constant": false,
        "inputs": [
            {
                "name": "_spender",
                "type": "address"
            },
            {
                "name": "_value",
                "type": "uint256"
            }
        ],
        "name": "approve",
        "outputs": [
            {
                "name": "",
                "type": "bool"
            }
        ],
        "payable": false,
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "constant": true,
        "inputs": [],
        "name": "totalSupply",
        "outputs": [
            {
                "name": "",
                "type": "uint256"
            }
        ],
        "payable": false,
        "stateMutability": "view",
        "type": "function"
    },
    {
        "constant": false,
        "inputs": [
            {
                "name": "_from",
                "type": "address"
            },
            {
                "name": "_to",
                "type": "address"
            },
            {
                "name": "_value",
                "type": "uint256"
            }
        ],
        "name": "transferFrom",
        "outputs": [
            {
                "name": "",
                "type": "bool"
            }
        ],
        "payable": false,
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "constant": true,
        "inputs": [],
        "name": "decimals",
        "outputs": [
            {
                "name": "",
                "type": "uint8"
            }
        ],
        "payable": false,
        "stateMutability": "view",
        "type": "function"
    },
    {
        "constant": true,
        "inputs": [
            {
                "name": "_owner",
                "type": "address"
            }
        ],
        "name": "balanceOf",
        "outputs": [
            {
                "name": "balance",
                "type": "uint256"
            }
        ],
        "payable": false,
        "stateMutability": "view",
        "type": "function"
    },
    {
        "constant": true,
        "inputs": [],
        "name": "symbol",
        "outputs": [
            {
                "name": "",
                "type": "string"
            }
        ],
        "payable": false,
        "stateMutability": "view",
        "type": "function"
    },
    {
        "constant": false,
        "inputs": [
            {
                "name": "_to",
                "type": "address"
            },
            {
                "name": "_value",
                "type": "uint256"
            }
        ],
        "name": "transfer",
        "outputs": [
            {
                "name": "",
                "type": "bool"
            }
        ],
        "payable": false,
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "constant": true,
        "inputs": [
            {
                "name": "_owner",
                "type": "address"
            },
            {
                "name": "_spender",
                "type": "address"
            }
        ],
        "name": "allowance",
        "outputs": [
            {
                "name": "",
                "type": "uint256"
            }
        ],
        "payable": false,
        "stateMutability": "view",
        "type": "function"
    },
    {
        "payable": true,
        "stateMutability": "payable",
        "type": "fallback"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "name": "owner",
                "type": "address"
            },
            {
                "indexed": true,
                "name": "spender",
                "type": "address"
            },
            {
                "indexed": false,
                "name": "value",
                "type": "uint256"
            }
        ],
        "name": "Approval",
        "type": "event"
    },
    {
        "anonymous": false,
        "inputs": [
            {
                "indexed": true,
                "name": "from",
                "type": "address"
            },
            {
                "indexed": true,
                "name": "to",
                "type": "address"
            },
            {
                "indexed": false,
                "name": "value",
                "type": "uint256"
            }
        ],
        "name": "Transfer",
        "type": "event"
    }
];

class ERC20 extends BaseContract  {
    constructor(address, abi = null){ 
		super(address, abi ?? erc20abi);		
    }

    async name(){
		if(this.contract){
			return this.contract.name();
		}
	}

    async symbol(){
		if(this.contract){
			return this.contract.symbol();
		}
	}

    async decimals(){
		if(this.contract){
			return this.contract.decimals();
		}
	}

    async totalSupply(){
		if(this.contract){
			return this.contract.totalSupply();
		}
	}

	async balanceOf(account){
		if(this.contract){
			return this.contract.balanceOf(account);
		}
	}

    async transfer(to, amount){
		if(this.contract){
			return this.contract
                    .connect(this.provider.getSigner())
                    .transfer(to, amount);
		}
	}

    async transferFrom(from, to, amount){
		if(this.contract){
			return this.contract
                    .connect(this.provider.getSigner())
                    .transferFrom(from, to, amount);
		}
	}

    async approve(spender, amount){
		if(this.contract){
			return this.contract
                    .connect(this.provider.getSigner())
                    .approve(spender, amount);
		}
	}

	async allowance(account, spender){
		if(this.contract){
			return this.contract.allowance(account, spender);
		}
	}
}

export { ERC20 }

function initializeContracts(){
	if (window.ethereum) {
		provider = new ethers.providers.Web3Provider(window.ethereum);
		signer = provider.getSigner();

		v2routerContract = new v2RouterContract();
		v2factoryContract = new v2FactoryContract();
	}
}

export {
	provider, signer,
	v2routerContract,
	v2factoryContract,
	initializeContracts
}



class v2FactoryContract extends BaseContract  {
    constructor(address = null){ 
		super(address ?? v2FactoryAddress, v2FactoryAbi);		
    }
}

export { v2FactoryContract }



class v2RouterContract extends BaseContract  {
    constructor(address = null){ 
		super(address ?? v2RouterAddress, v2RouterAbi);		
    }

    async amountsOut(amountIn, path){
        if(this.contract){
            return this.contract.getAmountsOut(amountIn, path);
        }
    }

    async SwapEthForToken(account, amountIn, amountOutMin, path)
    {
        if(this.contract){
            let options = { value: amountIn }
            return this.contract
                    .connect(this.provider.getSigner())
                    .swapExactETHForTokens(
                        amountOutMin,
                        path,
                        account,
                        Math.floor(Date.now() / 1000) + (60 * 10),
                        options
                    );
        }
    }

    async SwapTokenForEth(account, amountIn, amountOutMin, path)
    {
        if(this.contract){
            return this.contract
                .connect(this.provider.getSigner())
                .swapExactTokensForETH(
                    amountIn,
                    amountOutMin,
                    path,
                    account,
                    Math.floor(Date.now() / 1000) + (60 * 10)
            );
        }
    }

    async SwapTokenForEthwFee(account, amountIn, amountOutMin, path)
    {
        console.log("Swapping token for eth");
        if(this.contract){
            return this.contract
                .connect(this.provider.getSigner())
                .swapExactTokensForETHSupportingFeeOnTransferTokens(
                    amountIn,
                    amountOutMin,
                    path,
                    account,
                    Math.floor(Date.now() / 1000) + (60 * 10)
            );
        }
    }
}



async function transfer(contract, account, to, Id){
	if(contract && account){
		return contract.contract.connect(provider.getSigner())['safeTransferFrom(address,address,uint256)']
		(account, to, Id);
	}
}

export { v2RouterContract }
export { transfer }
export { polyChainID, polyScanBase, polyTestChainID, polyScanTestBase, targetChain }
export { tokens }

async function currentBlock(){
    return await provider.getBlockNumber();
}

async function nativeBalance(){
    return await provider.getBalance(currentAccount);
}

export { currentBlock, nativeBalance }

window.currentBlock = currentBlock;
function tobytes(input){
	return ethers.utils.toUtf8Bytes(input)
}

function tobytes32(input){
    return ethers.utils.formatBytes32String(input); 
}

function bytestoAscii(input){
    return ethers.utils.toUtf8String(input);
} 

function bytes32toAscii(input){
    return ethers.utils.parseBytes32String(input);
} 

//bignumber -> string
function fromWei(input, unit = "ether"){
	return ethers.utils.formatUnits(input, unit);
}

//parse string value as unit(default wei)
function asUnit(input, unit = "wei"){
	return ethers.utils.parseUnits(input, unit);
}

export { tobytes, tobytes32, bytestoAscii, bytes32toAscii, fromWei, asUnit }

window.tobytes = tobytes;
window.tobytes32 = tobytes32;
window.bytestoAscii = bytestoAscii;
window.bytes32toAscii = bytes32toAscii;
window.fromWei = fromWei;
window.asUnit = asUnit;


//Init
function initWalletEvents(){
    // account changed event
    //(accounts: Array<string>) => void
    window.ethereum.on('accountsChanged', accountsChanged);
 
    // chain changed event 
    //(chainId: string) => void
    window.ethereum.on('chainChanged', chainChanged);
}

//Eth events
function accountsChanged(accounts) {
   if (accounts.length === 0) {
      currentAccount =  null;
      window.dispatchEvent(onDisconnected);
   } 
   else {  
      if (accounts[0] !== currentAccount) { currentAccount = accounts[0]; }

      let onConnected = new CustomEvent("onConnected", { detail: currentAccount });
      window.dispatchEvent(onConnected);
   }

  //syncAccount();
}

function chainChanged(chainId) {
   console.log("chain changed");
   let onChainChanged = new CustomEvent("onChainChanged", { detail: currentChain });
   window.dispatchEvent(onChainChanged);

   //initializeContracts();
   //syncChain(chainId)
}

async function connect() 
{
   if (window.ethereum) 
   {
      let accounts = await ethereum
                              .request({ method: 'eth_requestAccounts' })
                              .catch((err) => 
                              {
                                 if (err.code === 4001) 
                                 {
                                    // EIP-1193 userRejectedRequest error
                                    // If this happens, the user rejected the connection request.
                                    console.log('Please connect to MetaMask.');
                                 } 
                                 else { console.error(err); }

                                 icon.src = 'img/MetaMask Fox Disabled.png';
                                 cnctTxt.innerText = "Connect";
                              });
      
      accountsChanged(accounts);
      //syncAccount();
      checkChain();    
   } 
   else { cnctTxt.innerText = "No wallet"; }
}

async function checkChain(){

   let chainId = await window.ethereum
                              .request({ method: 'eth_chainId' })
                              .catch((err) => { console.error(err); });
   //syncChain(chainId);
}

async function addtoken(contractAddress, contractSymbol, contractDecimals, contractImage){
   try {
      // wasAdded is a boolean. Like any RPC method, an error can be thrown.
      let wasAdded = await ethereum.request({
         method: 'wallet_watchAsset',
         params: {
            type: 'ERC20',
            options: {
            address: contractAddress, // The address of the token.
            symbol: contractSymbol, // A ticker symbol or shorthand, up to 5 characters.
            decimals: contractDecimals, // The number of decimals in the token.
            image: contractImage, // A string URL of the token logo.
            },
         },
      });
   
      if (wasAdded) {
         console.log('Thanks for your interest!');
      } else {
         console.log('Your loss!');
      }
   } catch (error) {
   console.log(error);
   }
}

export{ currentAccount, connect, initWalletEvents, accountsChanged }

window.connect = connect;
window.initWalletEvents = initWalletEvents;
window.onload = initWalletEvents;
window.accountsChanged = accountsChanged;


const v2FactoryAddress = "0xd8F4Db07f5a3F961e51641ED787468bEb3159999";
const v2FactoryAbi = [
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "_feeToSetter",
          "type": "address"
        }
      ],
      "payable": false,
      "stateMutability": "nonpayable",
      "type": "constructor"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "token0",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "token1",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "address",
          "name": "pair",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "name": "PairCreated",
      "type": "event"
    },
    {
      "constant": true,
      "inputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "name": "allPairs",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "payable": false,
      "stateMutability": "view",
      "type": "function"
    },
    {
      "constant": true,
      "inputs": [],
      "name": "allPairsLength",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "payable": false,
      "stateMutability": "view",
      "type": "function"
    },
    {
      "constant": false,
      "inputs": [
        {
          "internalType": "address",
          "name": "tokenA",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "tokenB",
          "type": "address"
        }
      ],
      "name": "createPair",
      "outputs": [
        {
          "internalType": "address",
          "name": "pair",
          "type": "address"
        }
      ],
      "payable": false,
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "constant": true,
      "inputs": [],
      "name": "feeTo",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "payable": false,
      "stateMutability": "view",
      "type": "function"
    },
    {
      "constant": true,
      "inputs": [],
      "name": "feeToSetter",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "payable": false,
      "stateMutability": "view",
      "type": "function"
    },
    {
      "constant": true,
      "inputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "name": "getPair",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "payable": false,
      "stateMutability": "view",
      "type": "function"
    },
    {
      "constant": false,
      "inputs": [
        {
          "internalType": "address",
          "name": "_feeTo",
          "type": "address"
        }
      ],
      "name": "setFeeTo",
      "outputs": [],
      "payable": false,
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "constant": false,
      "inputs": [
        {
          "internalType": "address",
          "name": "_feeToSetter",
          "type": "address"
        }
      ],
      "name": "setFeeToSetter",
      "outputs": [],
      "payable": false,
      "stateMutability": "nonpayable",
      "type": "function"
    }
];

export { v2FactoryAddress, v2FactoryAbi }
const v2RouterAddress = "0x6dda7853c78858cd623b06306ffADCEcdA80ec68";
const v2RouterAbi = [
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "_factory",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "_WETH",
        "type": "address"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "inputs": [],
    "name": "WETH",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "tokenA",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "tokenB",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amountADesired",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountBDesired",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountAMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountBMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "addLiquidity",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountA",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountB",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "amountTokenDesired",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountTokenMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETHMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "addLiquidityETH",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountToken",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETH",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      }
    ],
    "stateMutability": "payable",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "factory",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountOut",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserveIn",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserveOut",
        "type": "uint256"
      }
    ],
    "name": "getAmountIn",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountIn",
        "type": "uint256"
      }
    ],
    "stateMutability": "pure",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountIn",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserveIn",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserveOut",
        "type": "uint256"
      }
    ],
    "name": "getAmountOut",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountOut",
        "type": "uint256"
      }
    ],
    "stateMutability": "pure",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountOut",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      }
    ],
    "name": "getAmountsIn",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountIn",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      }
    ],
    "name": "getAmountsOut",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountA",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserveA",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserveB",
        "type": "uint256"
      }
    ],
    "name": "quote",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountB",
        "type": "uint256"
      }
    ],
    "stateMutability": "pure",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "tokenA",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "tokenB",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountAMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountBMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "removeLiquidity",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountA",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountB",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountTokenMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETHMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "removeLiquidityETH",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountToken",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETH",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountTokenMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETHMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "removeLiquidityETHSupportingFeeOnTransferTokens",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountETH",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountTokenMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETHMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      },
      {
        "internalType": "bool",
        "name": "approveMax",
        "type": "bool"
      },
      {
        "internalType": "uint8",
        "name": "v",
        "type": "uint8"
      },
      {
        "internalType": "bytes32",
        "name": "r",
        "type": "bytes32"
      },
      {
        "internalType": "bytes32",
        "name": "s",
        "type": "bytes32"
      }
    ],
    "name": "removeLiquidityETHWithPermit",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountToken",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETH",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "token",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountTokenMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountETHMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      },
      {
        "internalType": "bool",
        "name": "approveMax",
        "type": "bool"
      },
      {
        "internalType": "uint8",
        "name": "v",
        "type": "uint8"
      },
      {
        "internalType": "bytes32",
        "name": "r",
        "type": "bytes32"
      },
      {
        "internalType": "bytes32",
        "name": "s",
        "type": "bytes32"
      }
    ],
    "name": "removeLiquidityETHWithPermitSupportingFeeOnTransferTokens",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountETH",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "tokenA",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "tokenB",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "liquidity",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountAMin",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountBMin",
        "type": "uint256"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      },
      {
        "internalType": "bool",
        "name": "approveMax",
        "type": "bool"
      },
      {
        "internalType": "uint8",
        "name": "v",
        "type": "uint8"
      },
      {
        "internalType": "bytes32",
        "name": "r",
        "type": "bytes32"
      },
      {
        "internalType": "bytes32",
        "name": "s",
        "type": "bytes32"
      }
    ],
    "name": "removeLiquidityWithPermit",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "amountA",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountB",
        "type": "uint256"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountOut",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapETHForExactTokens",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "payable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountOutMin",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapExactETHForTokens",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "payable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountOutMin",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapExactETHForTokensSupportingFeeOnTransferTokens",
    "outputs": [],
    "stateMutability": "payable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountIn",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountOutMin",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapExactTokensForETH",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountIn",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountOutMin",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapExactTokensForETHSupportingFeeOnTransferTokens",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountIn",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountOutMin",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapExactTokensForTokens",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountIn",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountOutMin",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapExactTokensForTokensSupportingFeeOnTransferTokens",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountOut",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountInMax",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapTokensForExactETH",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "amountOut",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "amountInMax",
        "type": "uint256"
      },
      {
        "internalType": "address[]",
        "name": "path",
        "type": "address[]"
      },
      {
        "internalType": "address",
        "name": "to",
        "type": "address"
      },
      {
        "internalType": "uint256",
        "name": "deadline",
        "type": "uint256"
      }
    ],
    "name": "swapTokensForExactTokens",
    "outputs": [
      {
        "internalType": "uint256[]",
        "name": "amounts",
        "type": "uint256[]"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "stateMutability": "payable",
    "type": "receive"
  }
];

export { v2RouterAddress, v2RouterAbi }