import { secToTimeSpan } from "../time.js";

class Timer extends HTMLElement {  

    static get observedAttributes() {
        return ['end-time', 'timer-interval'];
    }

    root;
    get shadowRoot() { return this.root; }
    set shadowRoot(value) { this.root = value; }

    timer;
    get timerElement() { return this.timer; }
    set timerElement(value) { this.timer = value; }

    get endTime() { return this.getAttribute('end-time'); }
    set endTime(value) { this.setAttribute('end-time', value); }

    get interval() { return this.getAttribute('timer-interval'); }
    set interval(value) { this.setAttribute('timer-interval', value); }

    constructor(){ 
        super();
        
        this.root = this.attachShadow({ mode: "open" });
        this.timer = document.createElement("span");
        this.root.appendChild(this.timer);
    
        this.syncTime();
    }

    attributeChangedCallback(name, oldValue, newValue) {
        if (name === 'end-time' || name === 'timer-interval') {
            this.syncTime();
        }
    }

    syncTime(){
        this.updateTime();
        if(this.endTime && Date.now() < this.endTime){
            clearInterval(this.timerInterval);
            this.timerInterval = setInterval(this.updateTime.bind(this), this.interval ?? 1000);
        }
    }

    updateTime(){
        console.log("updating time");
        this.timer.textContent = !this.endTime || Date.now() >= this.endTime ? '00:00:00' : secToTimeSpan(Math.floor((this.endTime - Date.now())/1000));
    }

    start() { this.syncTime(); }
    stop(){ if(this.timerInterval) { clearInterval(this.timerInterval); } }
    
}

customElements.define('ohl-timer', Timer);